function [NODES,CG]=CGtoGRID(ini,def,cgwidth,nx,ny,nz,zviz,cutoff)
% HELP
% ---------- INPUT ----------
% ini [string] - initial configuration [.mat] (Coord - coordinates; Box - box size and shape; Data - results to be coarse-grained)
% def [string] - deformed configuration [.mat]
% cgwidth [double] - Coarse grain width - Coarse grain function:
%                               cgfun=exp(-dr^2/cgwidth^2)/((pi*cgwidth^2)^(3/2))
% nx, ny, nz [integer] - how many segments should we divide the whole (grid
%                                   point number = ni+1) [-]
% zviz [integer] - which slice should we calculate (if zviz = 0 we calculate all) [-]
% cutoff [double] - how much of the system should be dulicated for periodicity [A]
% ---------- OUTPUT ----------
% NODES.CoordINI  - particul coordinates of the initial sample
%      .CoordINI0 - particul coordinates in untilted configuration of the
%                   initial sample
%      .SP        - discrete quantities of the initial sample
%      .CoordDEF  - particul coordinates of the deformed sample
%      .CoordDEF0 - particul coordinates in untilted configuration of the
%                   deformed sample
%      .SF        - discrete quantities of the deformed sample
%      .U         - particle displacements
%      .NAU       - particle non-affice displacements
%      .NAUA      - particle non-affine displacement field
%
% CG.cgi          - ith coordinate of the grid in the initial cartesian coordinate
%                   system 
%   .dens         - coarse-grained density
%   .adens        - coarse-grained pzrticle density
%   .ui           - coarse-grained displacement in the ith direction
%   .cgNAi        - coarse-grained non-affine displacement in the ith direction
%   .cgNAA        - coarse-grained non-affine displacement amplitude
%   .Res0         - coarse-grained variables in the initial state
%   .Res          - coarse-grained variables in the deformed state
%   .dRes         - difference in variables between initial and deformed
%                   states
%   .eij          - ij strain

% Waitbar
w_b=waitbar(0,'Please wait...');
cfsqr=cutoff^2;

if zviz>nz+1
    disp('Error on zviz! It cannot be larger then nz!')
    return
end

cgwidthsq=cgwidth^2;

if zviz~=0
    kmin=1;
    kmax=1;
else
    kmin=1;
    kmax=nz+1;
end
tic
% ------------------------------------------------------------------------------------------
% Uploading initial systems
% Initial configuration
load(ini);
BoxINI=Box;
CoordINI=Coord;
SP=Data;

[Lx,Ly,Lz,xyp,xzp,yxp,x_min_p,x_max_p,y_min_p,y_max_p,z_min_p,z_max_p,xmin,xmax,ymin,ymax,zmin,zmax,aap,abo,avol,bb]=BoxProp(BoxINI);
nIN=length(CoordINI(:,1));   % size of original system at initial position
% Moving the coordinate system to left bottom corner from the middle
CoordINI=[CoordINI(:,1) CoordINI(:,2)-x_min_p CoordINI(:,3)-y_min_p CoordINI(:,4)-z_min_p CoordINI(:,5)];
NODES.CoordINI=CoordINI;
NODES.SP=SP;
% Calculating local coordinates
rp=CoordINI(:,2:4)*bb;
% Transformint box from triclinic to rectangular
CoordINI0(:,2:4)=rp*abo';
CoordINI0(:,[1,5])=CoordINI(:,[1,5]);
NODES.CoordINI0=CoordINI0;
%
disp('Initial system uploaded!')

% Deformed configuraton
load(def);
BoxDEF=Box;
CoordDEF=Coord;
SF=Data;
if isempty(SF)
    ndata=0;
else
    ndata=length(SF(1,:));  % number of variable to coarse-grain
end

[Lxf,Lyf,Lzf,xyf,xzf,yzf,x_min_f,x_max_f,y_min_f,y_max_f,z_min_f,z_max_f,xminf,xmaxf,yminf,ymaxf,zminf,zmaxf,aaf,abof,avolf,bbf]=BoxProp(BoxDEF);
nf=length(CoordDEF(:,1));   % size of original system at initial position
% Moving the coordinate system to left bottom corner from the middle
CoordDEF=[CoordDEF(:,1) CoordDEF(:,2)-x_min_f CoordDEF(:,3)-y_min_f CoordDEF(:,4)-z_min_f CoordDEF(:,5)];
NODES.CoordDEF=CoordDEF;   % output
NODES.SF=SF;
% Calculating local coordinates
rf=CoordDEF(:,2:4)*bbf;
% Puting atoms back in place which were moved by the perioduc boundaies
rf=rf-((rf-rp)>0.5)+((rf-rp)<-0.5);
CoordNew=rf*aaf';

CoordDEF=[CoordDEF(:,1) CoordNew(:,1:3) CoordDEF(:,5)];
Coord0(:,2:4)=rf*abof'; % In Lagrangian coordinates
Coord0(:,[1,5])=CoordDEF(:,[1,5]);
NODES.CoordDEF0=Coord0; % output
disp('Deformed system uploaded')
clear CoordNew
% Increasing system size to take periodicity into account
[rp,rf,idlist]=Period(rp,rf,cutoff/Lx);

CoordNew=[CoordDEF(idlist,1) rf*aaf' CoordDEF(idlist,5)];
CoordNewIN=[CoordINI(idlist,1) rp*aap' CoordINI(idlist,5)];
CoordDEF=CoordNew;
CoordINI=CoordNewIN;

Coord0=[CoordDEF(idlist,1) rf*abof' CoordDEF(idlist,5)];
CoordINI0=[CoordINI(idlist,1) rp*abo' CoordINI(idlist,5)];

if ndata>=1
    SP=SP(idlist,:);
    SF=SF(idlist,:);
end

disp('System size increased')

waitbar(0,w_b,'Input uploaded!');
% --------------------------------------------------------------------------------------------
% Non-affine displacements/atom
dna=rf-rp;
%dna=dna-(dna>0.5)+(dna<-0.5);   % periodic boundary conditions (just for security)
dCoord=dna*aaf';         % aap for Lagransian system (aaf for Euler)
dCoordA=sum(dCoord.^2,2).^0.5;  % amplitude of non-affine displacements
NODES.NAU=dCoord(1:nf,:);    % output
NODES.NAUA=sum(NODES.NAU.^2,2).^0.5;

%
% Total displacements/atom
Dist=CoordDEF(:,2:4)-CoordINI(:,2:4);
dr=Dist*bbf;
Dist=Dist-[Lxf*(Dist(:,1)>Lx/2) Lyf*(Dist(:,2)>Ly/2) Lzf*(Dist(:,3)>Lz/2)]+[Lx*(Dist(:,1)<-Lx/2) Ly*(Dist(:,2)<-Ly/2) Lz*(Dist(:,3)<-Lz/2)];    % periodic boundary conditions
NODES.U=Dist(1:nf,:);
%
% ----------------------------------------------------------------------------------------------
% Coarse-graining
% ----------------------------------------------------------------------------------------------
%
waitbar(0,w_b,'Start coarse-graining!');
tic
% Allocatng variables
% Grid coordinates
cgx=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgy=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgz=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Particle density
rho=zeros(nx+1,ny+1,(kmax-kmin)+1);
adens=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Displacements
ux=zeros(nx+1,ny+1,(kmax-kmin)+1);
uy=zeros(nx+1,ny+1,(kmax-kmin)+1);
uz=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Non-affinde displacements
cgNAx=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgNAy=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgNAz=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgNAA=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Strain increments
exx=zeros(nx+1,ny+1,(kmax-kmin)+1);
eyy=zeros(nx+1,ny+1,(kmax-kmin)+1);
ezz=zeros(nx+1,ny+1,(kmax-kmin)+1);
exy=zeros(nx+1,ny+1,(kmax-kmin)+1);
exz=zeros(nx+1,ny+1,(kmax-kmin)+1);
eyz=zeros(nx+1,ny+1,(kmax-kmin)+1);
% displacement*derivate of coarse grain function
sum_ux_dfdx=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_ux_dfdy=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_ux_dfdz=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uy_dfdx=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uy_dfdy=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uy_dfdz=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uz_dfdx=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uz_dfdy=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_uz_dfdz=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Derivate of coarse grain function
sum_dfdx=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_dfdy=zeros(nx+1,ny+1,(kmax-kmin)+1);
sum_dfdz=zeros(nx+1,ny+1,(kmax-kmin)+1);
% Discrete values at the initial state
if ndata>=1
    Res0=zeros(ndata,nx+1,ny+1,(kmax-kmin)+1);
    % Discrete values at the final state
    Res=zeros(ndata,nx+1,ny+1,(kmax-kmin)+1);
    % difference in discrete values
    dRes=zeros(ndata,nx+1,ny+1,(kmax-kmin)+1);
else
    Res0=[];
    Res=[];
    dRes=[];
end
cgx=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgy=zeros(nx+1,ny+1,(kmax-kmin)+1);
cgz=zeros(nx+1,ny+1,(kmax-kmin)+1);

cnt2=1;
% ------------------------------------------------------------
% Coarse-graining on grid
for i=1:nx+1
    for j=1:ny+1
        for k=kmin:kmax
            clear cddr cdxyz
            % Grid coordinates
            cgx(i,j,k)=xmin+(i-1)*Lxf/nx;
            cgy(i,j,k)=ymin+(j-1)*Lyf/ny;
            if zviz==0
                cgz(i,j,k)=zmin+(k-1)*Lzf/nz;
            else
                cgz(i,j,k)=zmin+(zviz)*Lzf/nz;
            end
            % XYZ distance between atoms and grid point
            cddxyz=[ones(length(Coord0(:,2)),1)*cgx(i,j,k) ones(length(Coord0(:,3)),1)*cgy(i,j,k) ones(length(Coord0(:,4)),1)*cgz(i,j,k)]-Coord0(:,2:4);
            cddxyz0=[ones(length(CoordINI0(:,2)),1)*cgx(i,j,k) ones(length(CoordINI0(:,3)),1)*cgy(i,j,k) ones(length(CoordINI0(:,4)),1)*cgz(i,j,k)]-CoordINI0(:,2:4);
            % Distance between atoms and grid point
            cddrsq=sum(cddxyz.^2,2);
            cddrsq0=sum(cddxyz0.^2,2);
            % Cutoff
            cddrsq0=cddrsq0(cddrsq<cfsqr);
            Coordbm=Coord0(cddrsq<cfsqr,5);
            Dist_loc=Dist(cddrsq<cfsqr,:);
            dCoord_loc=dCoord(cddrsq<cfsqr,:);
            dCoordA_loc=dCoordA(cddrsq<cfsqr,:);
            
            if ndata>=1
                SP_loc=SP(cddrsq<cfsqr,:);
                SF_loc=SF(cddrsq<cfsqr,:);
            end

            cddxyz=cddxyz(cddrsq<cfsqr,:);
            cddxyz0=cddxyz0(cddrsq<cfsqr,:);
            cddrsq=cddrsq(cddrsq<cfsqr);

            % Coarse grain weight function for each atom
            cgfun=exp(-cddrsq/cgwidthsq)/((pi*cgwidthsq)^(3/2));
            cgfun0=exp(-cddrsq0/cgwidthsq)/((pi*cgwidthsq)^(3/2));           
            % Atom density
            rho(i,j,k)=sum(Coordbm.*cgfun);
            adens(i,j,k)=sum(ones(length(Coordbm),1).*cgfun);
            % Displacement
            ux(i,j,k)=sum(Coordbm.*cgfun.*Dist_loc(:,1))/rho(i,j,k);
            uy(i,j,k)=sum(Coordbm.*cgfun.*Dist_loc(:,2))/rho(i,j,k);
            uz(i,j,k)=sum(Coordbm.*cgfun.*Dist_loc(:,3))/rho(i,j,k);
            % Non-affine displacement
            cgNAx(i,j,k)=sum(Coordbm.*cgfun.*dCoord_loc(:,1))/rho(i,j,k);
            cgNAy(i,j,k)=sum(Coordbm.*cgfun.*dCoord_loc(:,2))/rho(i,j,k);
            cgNAz(i,j,k)=sum(Coordbm.*cgfun.*dCoord_loc(:,3))/rho(i,j,k);
            cgNAA(i,j,k)=sum(Coordbm.*cgfun.*dCoordA_loc)/rho(i,j,k);
% Derivatve function for coarse grain
            dcgfundx=-2*(cddxyz(:,1)).*exp(-cddrsq/cgwidthsq)/((pi^(3/2))*cgwidth^5);
            dcgfundy=-2*(cddxyz(:,2)).*exp(-cddrsq/cgwidthsq)/((pi^(3/2))*cgwidth^5);
            dcgfundz=-2*(cddxyz(:,3)).*exp(-cddrsq/cgwidthsq)/((pi^(3/2))*cgwidth^5);
% Derivate sums of CG function weighted
            sum_dfdx(i,j,k)=sum(dcgfundx.*Coordbm);
            sum_dfdy(i,j,k)=sum(dcgfundy.*Coordbm);
            sum_dfdz(i,j,k)=sum(dcgfundz.*Coordbm);
            sum_ux_dfdx(i,j,k)=sum(Coordbm.*dcgfundx.*Dist_loc(:,1));
            sum_uy_dfdx(i,j,k)=sum(Coordbm.*dcgfundx.*Dist_loc(:,2));
            sum_uz_dfdx(i,j,k)=sum(Coordbm.*dcgfundx.*Dist_loc(:,3));
            sum_ux_dfdy(i,j,k)=sum(Coordbm.*dcgfundy.*Dist_loc(:,1));
            sum_uy_dfdy(i,j,k)=sum(Coordbm.*dcgfundy.*Dist_loc(:,2));
            sum_uz_dfdy(i,j,k)=sum(Coordbm.*dcgfundy.*Dist_loc(:,3));
            sum_ux_dfdz(i,j,k)=sum(Coordbm.*dcgfundz.*Dist_loc(:,1));
            sum_uy_dfdz(i,j,k)=sum(Coordbm.*dcgfundz.*Dist_loc(:,2));
            sum_uz_dfdz(i,j,k)=sum(Coordbm.*dcgfundz.*Dist_loc(:,3));
% Engineering strains
            exx(i,j,k)=sum_ux_dfdx(i,j,k)/rho(i,j,k)-ux(i,j,k)*sum_dfdx(i,j,k)/rho(i,j,k);
            eyy(i,j,k)=sum_uy_dfdy(i,j,k)/rho(i,j,k)-uy(i,j,k)*sum_dfdy(i,j,k)/rho(i,j,k);   
            ezz(i,j,k)=sum_uz_dfdz(i,j,k)/rho(i,j,k)-uz(i,j,k)*sum_dfdz(i,j,k)/rho(i,j,k);   
            exy(i,j,k)=((sum_ux_dfdy(i,j,k)+sum_uy_dfdx(i,j,k))/rho(i,j,k))-((ux(i,j,k)*sum_dfdy(i,j,k)+uy(i,j,k)*sum_dfdx(i,j,k))/rho(i,j,k));
            exz(i,j,k)=((sum_ux_dfdz(i,j,k)+sum_uz_dfdx(i,j,k))/rho(i,j,k))-((ux(i,j,k)*sum_dfdz(i,j,k)+uz(i,j,k)*sum_dfdx(i,j,k))/rho(i,j,k));
            eyz(i,j,k)=((sum_uz_dfdy(i,j,k)+sum_uy_dfdz(i,j,k))/rho(i,j,k))-((uz(i,j,k)*sum_dfdy(i,j,k)+uy(i,j,k)*sum_dfdz(i,j,k))/rho(i,j,k));
% -----------------------------------------------------------------------------------
            % Discrete values
            for cdata=1:ndata
                Res0(cdata,i,j,k)=sum(SP_loc(:,cdata).*cgfun);
                Res(cdata,i,j,k)=sum(SF_loc(:,cdata).*cgfun);
            end
            if mod(cnt2,ceil(((nx+1)*(ny+1)*(kmax-kmin+1))/1000))==0
                progr=(cnt2/((nx+1)*(ny+1)*(kmax-kmin+1)));
                waitbar(progr,w_b,sprintf('%1.1f %% coarse-graining is ready...',progr*100));
             end
            cnt2=cnt2+1;
        end
    end
end
disp('Coarse-graining took:')
toc
% Stress increment
dRes=Res-Res0;

% Converting molar mass and Angstroms to g/cm3
% (if applicable)
dens=rho/6e23*1e24;

% output
CG.cgx=cgx;
CG.cgy=cgy;
CG.cgz=cgz;
CG.dens=dens;
CG.adens=adens;
CG.ux=ux;
CG.uy=uy;
CG.uz=uz;
CG.cgNAx=cgNAx;
CG.cgNAy=cgNAy;
CG.cgNAz=cgNAz;
CG.cgNAA=cgNAA;
CG.Res0=Res0;
CG.Res=Res;
CG.dRes=dRes;
CG.exx=exx;
CG.eyy=eyy;
CG.ezz=ezz;
CG.exy=exy;
CG.exz=exz;
CG.eyz=eyz;

delete(w_b)
disp('Ready!');

%%%%%%%%%%%%%%% Box details %%%%%%%%%%%%%%%%%%%%%%%
function [Lx,Ly,Lz,xyp,xzp,yzp,xmin_p,xmax_p,ymin_p,ymax_p,zmin_p,zmax_p,xmin,xmax,ymin,ymax,zmin,zmax,aa,ab,avol,bb]=BoxProp(Box)
% Box properties
xmin_p=Box(1,1); xmax_p=Box(1,2); xyp=Box(1,3); Lx=xmax_p-xmin_p;
ymin_p=Box(2,1); ymax_p=Box(2,2); xzp=Box(2,3); Ly=ymax_p-ymin_p;
zmin_p=Box(3,1); zmax_p=Box(3,2); yzp=Box(3,3); Lz=zmax_p-zmin_p;
% aa(a,b): a=direction, b=no du vecteur.
aa(1,1)=Lx; aa(2,1)=0; aa(3,1)=0;
aa(1,2)=xyp; aa(2,2)=Ly; aa(3,2)=0;
aa(1,3)=xzp; aa(2,3)=yzp; aa(3,3)=Lz;
ab(1,1)=Lx; ab(2,1)=0; ab(3,1)=0;
ab(1,2)=0; ab(2,2)=Ly; ab(3,2)=0;
ab(1,3)=0; ab(2,3)=0; ab(3,3)=Lz;
xmin=0; xmax=Lx; ymin=0; ymax=Ly; zmin=0; zmax=Lz;
% Calculating Reciprocal lattice vectors
bb(:,1)=cross(aa(:,2),aa(:,3));
bb(:,2)=cross(aa(:,3),aa(:,1));
bb(:,3)=cross(aa(:,1),aa(:,2));
avol=aa(:,1)'*cross(aa(:,2),aa(:,3));
bb=bb/avol;

%%%%%%%%%%%%%%%%% ADDING PERIODICITY %%%%%%%%%%%%%%%%
function [rp,rf,id]=Period(rp,rf,cfrat)
% cutoff is a ratio: cutoff/Lx
id=accumarray([1:length(rp(:,1))]',1:length(rp(:,1)));
for ii=1:3
    clear Coord_p Coord_n
    Coord_p=rp(rp(:,ii)<cfrat,:);
    Coord2_p=rf(rp(:,ii)<cfrat,:);
    Coord_p(:,ii)=Coord_p(:,ii)+1;
    Coord2_p(:,ii)=Coord2_p(:,ii)+1;
    idp=id(find(rp(:,ii)<cfrat));
    Coord_n=rp(rp(:,ii)>(1-cfrat),:);
    idn=id(find(rp(:,ii)>(1-cfrat)));
    Coord2_n=rf(rp(:,ii)>(1-cfrat),:);
    Coord_n(:,ii)=Coord_n(:,ii)-1;
    Coord2_n(:,ii)=Coord2_n(:,ii)-1;
    rp=[rp;Coord_p;Coord_n];
    rf=[rf;Coord2_p;Coord2_n];
    id=[id;idp;idn];
end



